
/*
 *                            COPYRIGHT
 *
 *  XAnalyser, frequence spectrum analyser for X Windows
 *  Copyright (C) 1998 Arvin Schnell
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Contact addresses:
 *  arvin@informatik.uni-bremen.de
 *  Arvin Schnell, Am Heidberg 8, 28865 Lilienthal, Germany
 *
 */


#include <iostream.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <math.h>

#include "DSP.hh"



const char audio_name[] = "/dev/dsp";



int
DSP::open( DSP_MODE mode, int format, int stereo, int speed, int buffer_size = 0 )
{ 
  int openmode;
  
  switch( mode ) {
  case DSP_PLAY: openmode = O_WRONLY; break;
  case DSP_RECORD: openmode = O_RDONLY; break;
  default: return 0;
  }
  
  audio_fd = ::open( audio_name, openmode, 0 );
  if( audio_fd == -1 ) {
    cerr << "Can't open " << audio_name << ": "
	 << strerror( errno ) << '\n';
    return 0;
  }
  
  
  if( buffer_size != 0 ) {
    
    int s = (int)( floor( log( buffer_size ) / log( 2.0 ) + 0.5 ) );
    if( ( 1L << s ) != buffer_size ) {
      cerr << "Invalid requested buffer size\n";
      return 0;
    }
    
    int arg = 0x00020000 + s;
    if( ioctl( audio_fd, SNDCTL_DSP_SETFRAGMENT, &arg ) ) {
      cerr << "Can't set fragment size: " << strerror( errno ) << '\n';
      return 0;
    }
  }
  
  
  audio_format = format;
  if( ioctl( audio_fd, SNDCTL_DSP_SETFMT, &audio_format ) == -1 ) {
    cerr << strerror( errno ) << '\n';
    return 0;
  }
  
  if( audio_format != format ) {
    cerr << "Can't set requested audio format\n";
    return 0;
  }
  
  
  audio_stereo = stereo;
  if( ioctl( audio_fd, SNDCTL_DSP_STEREO, &audio_stereo ) == -1 ) {
    cerr << strerror( errno ) << '\n';
    return 0;
  }
  
  if( audio_stereo != stereo ) {
    cerr << "Can't set mono/stereo\n";
    return 0;
  }
  
  
  audio_speed = speed;
  if( ioctl( audio_fd, SNDCTL_DSP_SPEED, &audio_speed ) == -1 ) {
    cerr << strerror( errno ) << '\n';
    return 0;
  }
  
  if( audio_speed != speed ) {
    cerr << "Can't set requested speed\n";
    cerr << "requested speed: " << speed << ", actual speed: "
	 << audio_speed << '\n';
    return 0;
  }
  
  
  if( ioctl( audio_fd, SNDCTL_DSP_GETBLKSIZE, &audio_buffer_size ) == -1 ) {
    cerr << strerror( errno ) << '\n';
    return 0;
  }
  
  
  if( buffer_size != 0 && audio_buffer_size != buffer_size ) {
    cerr << "Can't set requested buffer size\n";
    return 0;
  }
  
  
  /*
  if( ioctl( audio_fd, SNDCTL_DSP_GETCAPS, &audio_caps ) == -1 ) {
    cerr << strerror( errno ) << '\n';
    return 0;
  }
  */
  
  
  return 1;
}



int
DSP::close()
{
  ::close( audio_fd );
  audio_fd = -1;
  
  return 1;
}


int
DSP::sync()
{
  ioctl( audio_fd, SNDCTL_DSP_SYNC, NULL );
  
  return 1;
}



void
DSP::status()
{
  if( audio_fd == -1 ) {
    cerr << "not open\n";
    return;
  }
  
  cerr << "device: " << audio_name << "\n";
  cerr << "format: " << audio_format << "\n";
  cerr << "stereo: " << ( ( audio_stereo == 1 ) ? "yes" : "no" ) << '\n';
  cerr << "speed: " << audio_speed << " Hz\n";
  cerr << "buffer size: " << audio_buffer_size << " bytes\n";
  
  /*
  cerr << "capacities: ";
  if( audio_caps & DSP_CAP_DUPLEX )   cerr << "DSP_CAP_DUPLEX ";
  if( audio_caps & DSP_CAP_REALTIME ) cerr << "DSP_CAP_REALTIME ";
  if( audio_caps & DSP_CAP_BATCH )    cerr << "DSP_CAP_BATCH ";
  if( audio_caps & DSP_CAP_TRIGGER )  cerr << "DSP_CAP_TRIGGER ";
  if( audio_caps & DSP_CAP_MMAP )     cerr << "DSP_CAP_MMAP ";
  stream << '\n';
  */
}



int
DSP::write( void *buffer, int count )
{
  if( ::write( audio_fd, buffer, count ) != count ) {
    cerr << strerror( errno ) << '\n';
    return 0;
  }
  
  return 1;
  
  
  /*
  
  int count = 0, c;
  int p = 0;
  
  count = bytes_per_sample() * num_samples;
  
  if( count > audio_buffer_size ) {
    cerr << _HERE_ << '\n';
    exit( 1 );
  }
  
  while( count ) {
    
    if( count > audio_buffer_size )
      c = audio_buffer_size;
    else
      c = count;
    
    if( write( audio_fd, &buffer[p], c ) != c )
      return 0;
    
    count -= c;
    p += c/2;
    
  }
  
  return 1;
  */
}



int
DSP::read( void *buffer, int count )
{
  if( ::read( audio_fd, buffer, count ) != count ) {
    cerr << strerror( errno ) << '\n';
    return 0;
  }
  
  return 1;
  
  
  /*
  
  int l;
  
  int count, c;
  int k = 0;
  
  count = bytes_per_sample() * num_samples;
  
  if( count > audio_buffer_size ) {
    cerr << _HERE_ << '\n';
    exit( 1 );
  }
  
  while( count ) {
    
    if( count > audio_buffer_size )
      c = audio_buffer_size;
    else
      c = count;
    
    if( ( l = read( audio_fd, &buffer[k], c ) ) == 0 ) {
      cerr << strerror( errno ) << '\n';
      cerr << _HERE_ << '\n';
      return 0;
    }
    
    count -= l;
    k += l/2;
    
  }
  
  return 1;
  */
}


/*
int
DSP::bytes_per_sample() const
{
  int ret = 1;
  
  if( audio_stereo )
    ret *= 2;
  
  if( audio_format == AFMT_S16_LE || audio_format == AFMT_S16_BE )
    ret *= 2;
  
  return ret;
}
*/


void
DSP::swapbuffer( void *buffer, int count )
{
  int I = count / 2;
  signed short *tmp_buffer = (signed short *) buffer;
  signed short tmp;
  
  for( int i = 0; i < I; i++ ) {
    tmp =  ( 0x00ff & tmp_buffer[i] ) << 8;
    tmp |= ( 0xff00 & tmp_buffer[i] ) >> 8;
    tmp_buffer[i] = tmp;
  }
}

